@@ -1,31 +1,34 @@
 import asyncio
-from contextlib import suppress
+import datetime
 import logging
 import threading
+import uuid
 import warnings
+from contextlib import suppress
+from inspect import isawaitable
+
 from tornado.ioloop import PeriodicCallback
 
 import dask.config
 from dask.utils import format_bytes
 
-from .adaptive import Adaptive
-
+from ..core import Status
 from ..utils import (
-    log_errors,
-    sync,
     Log,
     Logs,
-    thread_state,
     format_dashboard_link,
+    log_errors,
     parse_timedelta,
+    sync,
+    thread_state,
 )
-
+from .adaptive import Adaptive
 
 logger = logging.getLogger(__name__)
 
 
 class Cluster:
-    """ Superclass for cluster objects
+    """Superclass for cluster objects
 
     This class contains common functionality for Dask Cluster manager classes.
 
@@ -48,15 +51,23 @@ class Cluster:
     """
 
     _supports_scaling = True
+    name = None
 
-    def __init__(self, asynchronous):
+    def __init__(self, asynchronous, quiet=False, name=None):
         self.scheduler_info = {"workers": {}}
         self.periodic_callbacks = {}
         self._asynchronous = asynchronous
         self._watch_worker_status_comm = None
         self._watch_worker_status_task = None
+        self._cluster_manager_logs = []
+        self.quiet = quiet
+        self.scheduler_comm = None
 
-        self.status = "created"
+        if name is not None:
+            self.name = name
+        elif self.name is None:
+            self.name = str(uuid.uuid4())[:8]
+        self.status = Status.created
 
     async def _start(self):
         comm = await self.scheduler_comm.live_comm()
@@ -66,12 +77,15 @@ class Cluster:
         self._watch_worker_status_task = asyncio.ensure_future(
             self._watch_worker_status(comm)
         )
-        self.status = "running"
+        self.status = Status.running
 
     async def _close(self):
-        if self.status == "closed":
+        if self.status == Status.closed:
             return
 
+        with suppress(AttributeError):
+            self._adaptive.stop()
+
         if self._watch_worker_status_comm:
             await self._watch_worker_status_comm.close()
         if self._watch_worker_status_task:
@@ -83,19 +97,28 @@ class Cluster:
         if self.scheduler_comm:
             await self.scheduler_comm.close_rpc()
 
-        self.status = "closed"
+        self.status = Status.closed
 
     def close(self, timeout=None):
+        # If the cluster is already closed, we're already done
+        if self.status == Status.closed:
+            if self.asynchronous:
+                future = asyncio.Future()
+                future.set_result(None)
+                return future
+            else:
+                return
+
         with suppress(RuntimeError):  # loop closed during process shutdown
             return self.sync(self._close, callback_timeout=timeout)
 
     def __del__(self):
-        if self.status != "closed":
+        if self.status != Status.closed:
             with suppress(AttributeError, RuntimeError):  # during closing
                 self.loop.add_callback(self.close)
 
     async def _watch_worker_status(self, comm):
-        """ Listen to scheduler for updates on adding and removing workers """
+        """Listen to scheduler for updates on adding and removing workers"""
         while True:
             try:
                 msgs = await comm.read()
@@ -119,7 +142,7 @@ class Cluster:
             raise ValueError("Invalid op", op, msg)
 
     def adapt(self, Adaptive=Adaptive, **kwargs) -> Adaptive:
-        """ Turn on adaptivity
+        """Turn on adaptivity
 
         For keyword arguments see dask.distributed.Adaptive
 
@@ -136,11 +159,11 @@ class Cluster:
         return self._adaptive
 
     def scale(self, n: int) -> None:
-        """ Scale cluster to n workers
+        """Scale cluster to n workers
 
         Parameters
         ----------
-        n: int
+        n : int
             Target number of workers
 
         Examples
@@ -168,9 +191,30 @@ class Cluster:
         else:
             return sync(self.loop, func, *args, **kwargs)
 
-    async def _get_logs(self, scheduler=True, workers=True):
+    def _log(self, log):
+        """Log a message.
+
+        Output a message to the user and also store for future retrieval.
+
+        For use in subclasses where initialisation may take a while and it would
+        be beneficial to feed back to the user.
+
+        Examples
+        --------
+        >>> self._log("Submitted job X to batch scheduler")
+        """
+        self._cluster_manager_logs.append((datetime.datetime.now(), log))
+        if not self.quiet:
+            print(log)
+
+    async def _get_logs(self, cluster=True, scheduler=True, workers=True):
         logs = Logs()
 
+        if cluster:
+            logs["Cluster"] = Log(
+                "\n".join(line[1] for line in self._cluster_manager_logs)
+            )
+
         if scheduler:
             L = await self.scheduler_comm.get_logs()
             logs["Scheduler"] = Log("\n".join(line for level, line in L))
@@ -182,11 +226,13 @@ class Cluster:
 
         return logs
 
-    def get_logs(self, scheduler=True, workers=True):
-        """ Return logs for the scheduler and workers
+    def get_logs(self, cluster=True, scheduler=True, workers=True):
+        """Return logs for the cluster, scheduler and workers
 
         Parameters
         ----------
+        cluster : boolean
+            Whether or not to collect logs for the cluster manager
         scheduler : boolean
             Whether or not to collect logs for the scheduler
         workers : boolean or Iterable[str], optional
@@ -199,7 +245,9 @@ class Cluster:
             A dictionary of logs, with one item for the scheduler and one for
             each worker
         """
-        return self.sync(self._get_logs, scheduler=scheduler, workers=workers)
+        return self.sync(
+            self._get_logs, cluster=cluster, scheduler=scheduler, workers=workers
+        )
 
     def logs(self, *args, **kwargs):
         warnings.warn("logs is deprecated, use get_logs instead", DeprecationWarning)
@@ -258,14 +306,14 @@ class Cluster:
         return text
 
     def _widget(self):
-        """ Create IPython widget for display within a notebook """
+        """Create IPython widget for display within a notebook"""
         try:
             return self._cached_widget
         except AttributeError:
             pass
 
         try:
-            from ipywidgets import Layout, VBox, HBox, IntText, Button, HTML, Accordion
+            from ipywidgets import HTML, Accordion, Button, HBox, IntText, Layout, VBox
         except ImportError:
             self._cached_widget = None
             return None
@@ -344,8 +392,9 @@ class Cluster:
         else:
             dashboard = "Not Available"
         return (
-            "<div style='background-color: #f2f2f2; display: inline-block; "
-            "padding: 10px; border: 1px solid #999999;'>\n"
+            "<div style='color: var(--jp-ui-font-color0, #000000); "
+            "background-color: var(--jp-layout-color2, #f2f2f2); display: inline-block; "
+            "padding: 10px; border: 1px solid var(--jp-border-color0, #999999);'>\n"
             "  <h3>{cls}</h3>\n"
             "  <ul>\n"
             "    <li><b>Dashboard: </b>{dashboard}\n"
@@ -374,7 +423,9 @@ class Cluster:
         return self
 
     async def __aexit__(self, typ, value, traceback):
-        await self.close()
+        f = self.close()
+        if isawaitable(f):
+            await f
 
     @property
     def scheduler_address(self):
@@ -385,8 +436,9 @@ class Cluster:
         return getattr(self, "_name", type(self).__name__)
 
     def __repr__(self):
-        text = "%s(%r, workers=%d, threads=%d" % (
+        text = "%s(%s, %r, workers=%d, threads=%d" % (
             self._cluster_class_name,
+            self.name,
             self.scheduler_address,
             len(self.scheduler_info["workers"]),
             sum(w["nthreads"] for w in self.scheduler_info["workers"].values()),
@@ -410,3 +462,9 @@ class Cluster:
     @property
     def observed(self):
         return {d["name"] for d in self.scheduler_info["workers"].values()}
+
+    def __eq__(self, other):
+        return self.name == other.name
+
+    def __hash__(self):
+        return id(self)
