@@ -1,10 +1,17 @@
 import logging
+import os
+import socket
+import subprocess
+import sys
+import uuid
+
+from dask.utils import funcname
 
 logger = logging.getLogger(__name__)
 
 
 class SchedulerPlugin:
-    """ Interface to extend the Scheduler
+    """Interface to extend the Scheduler
 
     The scheduler operates by triggering and responding to events like
     ``task_finished``, ``update_graph``, ``task_erred``, etc..
@@ -38,14 +45,14 @@ class SchedulerPlugin:
     """
 
     async def start(self, scheduler):
-        """ Run when the scheduler starts up
+        """Run when the scheduler starts up
 
         This runs at the end of the Scheduler startup process
         """
         pass
 
     async def close(self):
-        """ Run when the scheduler closes down
+        """Run when the scheduler closes down
 
         This runs at the beginning of the Scheduler shutdown process, but after
         workers have been asked to shut down gracefully
@@ -53,41 +60,41 @@ class SchedulerPlugin:
         pass
 
     def update_graph(self, scheduler, dsk=None, keys=None, restrictions=None, **kwargs):
-        """ Run when a new graph / tasks enter the scheduler """
+        """Run when a new graph / tasks enter the scheduler"""
 
     def restart(self, scheduler, **kwargs):
-        """ Run when the scheduler restarts itself """
+        """Run when the scheduler restarts itself"""
 
     def transition(self, key, start, finish, *args, **kwargs):
-        """ Run whenever a task changes state
+        """Run whenever a task changes state
 
         Parameters
         ----------
-        key: string
-        start: string
+        key : string
+        start : string
             Start state of the transition.
             One of released, waiting, processing, memory, error.
-        finish: string
+        finish : string
             Final state of the transition.
-        *args, **kwargs: More options passed when transitioning
+        *args, **kwargs : More options passed when transitioning
             This may include worker ID, compute time, etc.
         """
 
     def add_worker(self, scheduler=None, worker=None, **kwargs):
-        """ Run when a new worker enters the cluster """
+        """Run when a new worker enters the cluster"""
 
     def remove_worker(self, scheduler=None, worker=None, **kwargs):
-        """ Run when a worker leaves the cluster """
+        """Run when a worker leaves the cluster"""
 
     def add_client(self, scheduler=None, client=None, **kwargs):
-        """ Run when a new client connects """
+        """Run when a new client connects"""
 
     def remove_client(self, scheduler=None, client=None, **kwargs):
-        """ Run when a client disconnects """
+        """Run when a client disconnects"""
 
 
 class WorkerPlugin:
-    """ Interface to extend the Worker
+    """Interface to extend the Worker
 
     A worker plugin enables custom code to run at different stages of the Workers'
     lifecycle: at setup, during task state transitions, when a task or dependency
@@ -127,7 +134,7 @@ class WorkerPlugin:
         """
 
     def teardown(self, worker):
-        """ Run when the worker to which the plugin is attached to is closed """
+        """Run when the worker to which the plugin is attached to is closed"""
 
     def transition(self, key, start, finish, **kwargs):
         """
@@ -140,13 +147,13 @@ class WorkerPlugin:
 
         Parameters
         ----------
-        key: string
-        start: string
+        key : string
+        start : string
             Start state of the transition.
             One of waiting, ready, executing, long-running, memory, error.
-        finish: string
+        finish : string
             Final state of the transition.
-        kwargs: More options passed when transitioning
+        kwargs : More options passed when transitioning
         """
 
     def release_key(self, key, state, cause, reason, report):
@@ -155,28 +162,130 @@ class WorkerPlugin:
 
         Parameters
         ----------
-        key: string
-        state: string
+        key : string
+        state : string
             State of the released task.
             One of waiting, ready, executing, long-running, memory, error.
-        cause: string or None
+        cause : string or None
             Additional information on what triggered the release of the task.
-        reason: None
+        reason : None
             Not used.
-        report: bool
+        report : bool
             Whether the worker should report the released task to the scheduler.
         """
 
-    def release_dep(self, dep, state, report):
-        """
-        Called when the worker releases a dependency.
 
-        Parameters
-        ----------
-        dep: string
-        state: string
-            State of the released dependency.
-            One of waiting, flight, memory.
-        report: bool
-            Whether the worker should report the released dependency to the scheduler.
+def _get_worker_plugin_name(plugin) -> str:
+    """Returns the worker plugin name. If plugin has no name attribute
+    a random name is used."""
+    if hasattr(plugin, "name"):
+        return plugin.name
+    else:
+        return funcname(plugin) + "-" + str(uuid.uuid4())
+
+
+class PipInstall(WorkerPlugin):
+    """A Worker Plugin to pip install a set of packages
+
+    This accepts a set of packages to install on all workers.
+    You can also optionally ask for the worker to restart itself after
+    performing this installation.
+
+    .. note::
+
+       This will increase the time it takes to start up
+       each worker. If possible, we recommend including the
+       libraries in the worker environment or image. This is
+       primarily intended for experimentation and debugging.
+
+       Additional issues may arise if multiple workers share the same
+       file system. Each worker might try to install the packages
+       simultaneously.
+
+    Parameters
+    ----------
+    packages : List[str]
+        A list of strings to place after "pip install" command
+    pip_options : List[str]
+        Additional options to pass to pip.
+    restart : bool, default False
+        Whether or not to restart the worker after pip installing
+        Only functions if the worker has an attached nanny process
+
+    Examples
+    --------
+    >>> from dask.distributed import PipInstall
+    >>> plugin = PipInstall(packages=["scikit-learn"], pip_options=["--upgrade"])
+
+    >>> client.register_worker_plugin(plugin)
+    """
+
+    name = "pip"
+
+    def __init__(self, packages, pip_options=None, restart=False):
+        self.packages = packages
+        self.restart = restart
+        if pip_options is None:
+            pip_options = []
+        self.pip_options = pip_options
+
+    async def setup(self, worker):
+        from ..lock import Lock
+
+        async with Lock(socket.gethostname()):  # don't clobber one installation
+            logger.info("Pip installing the following packages: %s", self.packages)
+            proc = subprocess.Popen(
+                [sys.executable, "-m", "pip", "install"]
+                + self.pip_options
+                + self.packages,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+            )
+            stdout, stderr = proc.communicate()
+            returncode = proc.wait()
+
+            if returncode:
+                logger.error("Pip install failed with '%s'", stderr.decode().strip())
+                return
+
+            if self.restart and worker.nanny:
+                lines = stdout.strip().split(b"\n")
+                if not all(
+                    line.startswith(b"Requirement already satisfied") for line in lines
+                ):
+                    worker.loop.add_callback(
+                        worker.close_gracefully, restart=True
+                    )  # restart
+
+
+# Adapted from https://github.com/dask/distributed/issues/3560#issuecomment-596138522
+class UploadFile(WorkerPlugin):
+    """A WorkerPlugin to upload a local file to workers.
+
+    Parameters
+    ----------
+    filepath: str
+        A path to the file (.py, egg, or zip) to upload
+
+    Examples
+    --------
+    >>> from distributed.diagnostics.plugin import UploadFile
+
+    >>> client.register_worker_plugin(UploadFile("/path/to/file.py"))  # doctest: +SKIP
+    """
+
+    name = "upload_file"
+
+    def __init__(self, filepath):
         """
+        Initialize the plugin by reading in the data from the given file.
+        """
+        self.filename = os.path.basename(filepath)
+        with open(filepath, "rb") as f:
+            self.data = f.read()
+
+    async def setup(self, worker):
+        response = await worker.upload_file(
+            comm=None, filename=self.filename, data=self.data, load=True
+        )
+        assert len(self.data) == response["nbytes"]
