@@ -5,6 +5,11 @@ import psutil
 from .compatibility import WINDOWS
 from .metrics import time
 
+try:
+    from .diagnostics import nvml
+except Exception:
+    nvml = None
+
 
 class SystemMonitor:
     def __init__(self, n=10000):
@@ -34,6 +39,14 @@ class SystemMonitor:
             self.num_fds = deque(maxlen=n)
             self.quantities["num_fds"] = self.num_fds
 
+        if nvml is not None:
+            self.gpu_name = None
+            self.gpu_memory_total = None
+            self.gpu_utilization = deque(maxlen=n)
+            self.gpu_memory_used = deque(maxlen=n)
+            self.quantities["gpu_utilization"] = self.gpu_utilization
+            self.quantities["gpu_memory_used"] = self.gpu_memory_used
+
         self.update()
 
     def recent(self):
@@ -77,6 +90,18 @@ class SystemMonitor:
             self.num_fds.append(num_fds)
             result["num_fds"] = num_fds
 
+        # give external modules (like dask-cuda) a chance to initialize CUDA context
+        if nvml is not None and nvml.nvmlInit is not None:
+            if self.gpu_name is None:
+                gpu_extra = nvml.one_time()
+                self.gpu_name = gpu_extra["name"]
+                self.gpu_memory_total = gpu_extra["memory-total"]
+            gpu_metrics = nvml.real_time()
+            self.gpu_utilization.append(gpu_metrics["utilization"])
+            self.gpu_memory_used.append(gpu_metrics["memory-used"])
+            result["gpu_utilization"] = gpu_metrics["utilization"]
+            result["gpu_memory_used"] = gpu_metrics["memory-used"]
+
         return result
 
     def __repr__(self):
